<!DOCTYPE gui>
<gui name="kcop" version="2">
<MenuBar>
  <Menu name="execute"><text>&amp;Extra</text>
    <Action name="reload"/>
    <Action name="execute"/>
    <Action name="langmode"/>
  </Menu>
</MenuBar>
<ToolBar fullWidth="true" name="mainToolBar"><text>Main Toolbar</text>
    <Action name="reload"/>
    <Action name="execute"/>
    <Action name="langmode"/>
</ToolBar>
</gui>
